"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reactSSROutputTarget = void 0;
const compiler_1 = require("@stencil/core/compiler");
const esbuild = __importStar(require("esbuild"));
const node_assert_1 = __importDefault(require("node:assert"));
const node_child_process_1 = require("node:child_process");
const node_fs_1 = require("node:fs");
const node_module_1 = require("node:module");
const node_path_1 = __importDefault(require("node:path"));
const reactSSROutputTarget = (config = {
    outPath: 'dist/react-components-ssr',
    type: 'server-only',
}) => ({
    type: 'custom',
    name: 'react-ssr-library',
    async generator(stencilConfig, _, buildContext) {
        const { logger, flags } = buildContext.config;
        if (stencilConfig.watch) {
            logger.warn('The react-ssr-library output target does not support watch mode at the moment. Please run `stencil build` instead if you want to generate the React SSR components.');
            return;
        }
        const { rootDir, fsNamespace } = stencilConfig;
        (0, node_assert_1.default)(rootDir, 'rootDir is not defined');
        (0, node_assert_1.default)(fsNamespace, 'fsNamespace is not defined');
        const { debug } = flags;
        const timespan = buildContext.createTimeSpan(`generate react started`, true);
        const outputDir = node_path_1.default.join(rootDir, config.outPath ?? '');
        const distDir = node_path_1.default.join(outputDir, 'dist');
        const esmDir = node_path_1.default.join(distDir, 'esm');
        const typesDir = node_path_1.default.join(distDir, 'types');
        (0, node_fs_1.rmSync)(distDir, { recursive: true, force: true });
        (0, node_fs_1.mkdirSync)(node_path_1.default.join(esmDir, 'components'), { recursive: true });
        const compiler = await (0, compiler_1.createCompiler)({
            ...stencilConfig,
            outputTargets: [
                { type: 'dist-custom-elements', dir: node_path_1.default.join(esmDir, 'components') },
                { type: 'dist-hydrate-script', empty: true, dir: node_path_1.default.join(esmDir, 'hydrate') },
            ],
        });
        await compiler.build();
        await compiler.destroy();
        (0, node_fs_1.cpSync)(node_path_1.default.join(rootDir, 'dist', 'types'), typesDir, { recursive: true });
        (0, node_fs_1.rmSync)(node_path_1.default.join(esmDir, 'hydrate', 'package.json'), { force: true });
        exec('tsx', [
            node_path_1.default.join(__dirname, 'scripts', 'build.ts'),
            '--dist-root',
            esmDir,
            '--components-prefix',
            fsNamespace.split('-').shift() ?? '',
        ], !!debug);
        exec('tsc', [
            '--project',
            node_path_1.default.join(__dirname, 'tsconfig.plugin.json'),
            '--rootDir',
            __dirname,
            '--outDir',
            esmDir,
            '--declarationDir',
            typesDir,
        ], !!debug);
        await esbuild.build({
            entryPoints: [node_path_1.default.join(esmDir, '*.js'), node_path_1.default.join(esmDir, '**', '*.js'), node_path_1.default.join(esmDir, '**', '**', '*.js')],
            format: 'cjs',
            logLevel: 'error',
            outdir: node_path_1.default.join(distDir, 'cjs'),
        });
        createPackageJson(outputDir, config.type === 'server-only', config.package ?? {});
        timespan.finish(`generate react finished`);
    },
});
exports.reactSSROutputTarget = reactSSROutputTarget;
const exec = (command, args, debug = false) => {
    const { error } = (0, node_child_process_1.spawnSync)(command, args, { stdio: debug ? 'inherit' : 'ignore' });
    if (error) {
        throw error;
    }
};
const createPackageJson = (outputDir, patchStencil = false, { author = 'Stencil React SSR', license = 'ISC', name = 'stencil-react-ssr', version = '0.0.0' }) => {
    const outputPackageJsonPath = node_path_1.default.join(outputDir, 'package.json');
    const require = (0, node_module_1.createRequire)(__filename);
    const templatePackageJson = require(node_path_1.default.join(__dirname, 'scripts', 'templates', 'package.json'));
    if ((0, node_fs_1.existsSync)(outputPackageJsonPath)) {
        const currentPackageJson = require(outputPackageJsonPath);
        (0, node_fs_1.writeFileSync)(outputPackageJsonPath, JSON.stringify(Object.assign(templatePackageJson, currentPackageJson), null, 2));
    }
    else {
        templatePackageJson.name = name;
        templatePackageJson.version = version;
        templatePackageJson.author = author;
        templatePackageJson.license = license;
        if (patchStencil) {
            templatePackageJson.dependencies.linkedom = '0.16.1';
            templatePackageJson.dependencies['@stencil/core'] = 'github:smartive/stencil-patched#v4.19.2';
        }
        else {
            templatePackageJson.dependencies['@stencil/core'] = '^4.19.2';
        }
        (0, node_fs_1.writeFileSync)(outputPackageJsonPath, JSON.stringify(templatePackageJson, null, 2));
    }
};
//# sourceMappingURL=index.js.map